
import java.io.ByteArrayOutputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

/**
 * web temperature setter
 *
 * @author matecsaba
 */
public class temper implements Runnable {

    /**
     * this is needed for cli startup
     *
     * @param args command line parameters
     */
    public static void main(String[] args) {
        temper app = new temper();
        String a;
        try {
            ByteArrayOutputStream buf = new ByteArrayOutputStream();
            a = "" + app.getClass().getName();
            a = app.httpRequest("http://localhost/" + a, "./" + a, "cli", "clibrowser", args, buf);
            a = "type=" + a + "\r\ndata:\r\n" + buf.toString();
        } catch (Exception e) {
            a = "exception " + e.getMessage();
        }
        System.out.println(a);
    }

    /**
     * where i'm located in ram
     */
    protected static temper staticTemper = null;

    /**
     * do one request
     *
     * @param url url of app
     * @param path path of app
     * @param peer client address
     * @param agent user agent
     * @param par parameters
     * @param buf result buffer, if empty, pathname must present
     * @return [pathname"][file name.]extension
     * @throws Exception if something went wrong
     */
    public static String httpRequest(String url, String path, String peer,
            String agent, String[] par, ByteArrayOutputStream buf)
            throws Exception {
        url = new URL(url).getPath();
        if (staticTemper == null) {
            staticTemper = new temper();
            staticTemper.path = path.substring(0, path.lastIndexOf("."));
            staticTemper.url = url;
            new Thread(staticTemper).start();
        }
        staticTemper.doRequest(par, buf, peer);
        return "html";
    }

    /**
     * where i'm located on host
     */
    protected String path;

    /**
     * where i'm located on net
     */
    protected String url;

    /**
     * reading store
     */
    protected temperData measDat[];

    /**
     * reading limit
     */
    protected int measNum;
    /**
     * reading used
     */
    protected int measUse;

    /**
     * current value: 0=standby, 1=heating, 2=door
     */
    protected int currValue = 0x10000;

    /**
     * last needed temperature
     */
    protected int lastNeeded;

    /**
     * time needed temperature
     */
    protected long timeNeeded;

    /**
     * time heating temperature
     */
    protected long timeHeating;

    /**
     * log file
     */
    protected String logFile;

    /**
     * door code
     */
    protected List<String> doorCode = new ArrayList<String>();

    /**
     * door time
     */
    protected int doorTime = 100;

    /**
     * minimum value
     */
    protected int minValue = 10;

    /**
     * maximum value
     */
    protected int maxValue = 30;

    /**
     * last setter peer
     */
    protected String lastSetter = "nobody";

    private synchronized void setValue(int val) {
        if (currValue == val) {
            return;
        }
        try {
            Runtime rtm = Runtime.getRuntime();
            String[] cmd = new String[2];
            cmd[0] = "./tempset.sh";
            cmd[1] = val + "";
            Process prc = rtm.exec(cmd);
            prc.waitFor();
        } catch (Exception e) {
            return;
        }
        currValue = val;
        timeHeating = temperUtil.getTime();
    }

    private void rangeCheck() {
        if (lastNeeded > maxValue) {
            lastNeeded = maxValue;
        }
        if (lastNeeded < minValue) {
            lastNeeded = minValue;
        }
    }

    private int doCalc() {
        for (int i = 0; i < measNum; i++) {
            measDat[i].getValue();
            measDat[i].doCalc(this);
        }
        measUse = -1;
        boolean win = false;
        for (int i = 0; i < measNum; i++) {
            if (!measDat[i].isWorking) {
                continue;
            }
            win |= measDat[i].isWindow;
            if (measUse >= 0) {
                continue;
            }
            measUse = i;
        }
        int i = currValue & 2;
        if (win) {
            return i;
        }
        if (measUse < 0) {
            return i;
        }
        return i | measDat[measUse].lastCalc;
    }

    private synchronized void writeLog(String who) {
        if (who == null) {
            who = lastSetter;
        }
        String a = "";
        for (int i = 0; i < measNum; i++) {
            a = a + ";" + measDat[i].getLog();
        }
        temperUtil.append(logFile, temperUtil.getTime() + ";" + who + ";" + currValue + ";" + lastNeeded + ";" + measUse + a);
    }

    public void run() {
        logFile = path + ".log";
        readConfig();
        measDat = new temperData[measNum];
        for (int i = 0; i < measNum; i++) {
            measDat[i] = new temperData(i + 1);
        }
        measDat[0].getValue();
        lastNeeded = measDat[0].lastMeasure;
        timeNeeded = temperUtil.getTime();
        lastSetter = "boot";
        rangeCheck();
        setValue(doCalc());
        writeLog("<boot>");
        for (;;) {
            temperUtil.sleep(60 * 1000);
            rangeCheck();
            setValue(doCalc());
            writeLog(null);
        }
    }

    private void readConfig() {
        List<String> c = temperUtil.readup(path + ".cfg");
        if (c == null) {
            return;
        }
        if (c.size() < 1) {
            return;
        }
        doorCode = temperUtil.readup(c.get(0));
        if (doorCode == null) {
            doorCode = new ArrayList<String>();
        }
        doorTime = temperUtil.str2num(c.get(1));
        logFile = c.get(2);
        minValue = temperUtil.str2num(c.get(3));
        maxValue = temperUtil.str2num(c.get(4));
        measNum = temperUtil.str2num(c.get(5));
    }

    /**
     * do one request
     *
     * @param par parameters
     * @param buf buffer to use
     * @param peer address
     * @return 1 on html result
     * @throws Exception on error
     */
    public synchronized int doRequest(String[] par, ByteArrayOutputStream buf, String peer) throws Exception {
        String tmp = "";
        String cmd = "";
        for (int i = 0; i < par.length; i++) {
            String a = par[i];
            int o = a.indexOf("=");
            if (o < 1) {
                continue;
            }
            String b = a.substring(0, o);
            a = a.substring(o + 1, a.length());
            if (b.equals("temp")) {
                tmp = a;
            }
            if (b.equals("cmd")) {
                cmd = a;
            }
        }
        if (cmd.equals("heat")) {
            lastNeeded = temperUtil.str2num(tmp);
            timeNeeded = temperUtil.getTime();
            lastSetter = peer;
            rangeCheck();
        }
        if (cmd.equals("door")) {
            boolean b = false;
            for (int i = 0; i < doorCode.size(); i++) {
                b |= tmp.equals(doorCode.get(i));
                if (b) {
                    break;
                }
            }
            String a = "bad code";
            if (b) {
                setValue((currValue & 1) | 2);
                writeLog(peer);
                temperUtil.sleep(doorTime);
                setValue(currValue & 1);
                writeLog(peer);
                a = "door opened";
            }
            buf.write("<html><head><title>door</title><body>".getBytes());
            buf.write(a.getBytes());
            buf.write("</body></html>".getBytes());
            return 1;
        }
        rangeCheck();
        String a = "<html><head><title>temper</title>";
        buf.write(a.getBytes());
        a = "<meta http-equiv=refresh content=\"30;url=" + url + "\"></head>";
        buf.write(a.getBytes());
        a = "<body bgcolor=\"#000000\" text=\"#00FF00\" link=\"#00FFFF\" vlink=\"#00FFFF\" alink=\"#00FFFF\">";
        buf.write(a.getBytes());
        long tim = temperUtil.getTime();
        for (int i = 0; i < measNum; i++) {
            a = measDat[i].getMeas() + "<br/>";
            buf.write(a.getBytes());
        }
        a = "needed: " + lastNeeded + " celsius, " + temperUtil.timePast(tim, timeNeeded) + " ago by " + lastSetter + "<br/>";
        buf.write(a.getBytes());
        for (int i = 0; i < measNum; i++) {
            a = measDat[i].getWindow();
            if (a == null) {
                continue;
            }
            a = a + "<br/>";
            buf.write(a.getBytes());
        }
        a = "heating: " + currValue + ", " + temperUtil.timePast(tim, timeHeating) + " ago, using #" + (measUse + 1) + "<br/>";
        buf.write(a.getBytes());
        a = "<form action=\"" + url + "\" method=get>wish: <input type=text name=temp value=\"" + lastNeeded + "\"> celsius";
        buf.write(a.getBytes());
        buf.write("<input type=submit name=cmd value=\"heat\">".getBytes());
        buf.write("</form><br/>".getBytes());
        for (int i = -5; i <= 5; i++) {
            int o = i + lastNeeded;
            a = "((<a href=\"" + url + "?temp=" + o + "&cmd=heat\">" + o + "</a>))";
            buf.write(a.getBytes());
        }
        buf.write("</body></html>".getBytes());
        return 1;
    }

}
